unit Unit1; 

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    ButtonsData: array [0..19] of TButton;
    procedure GenerateButtons;
    procedure ReleaseButtons;
    procedure ButtonClick(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

function ButtonSubclassProc(hWnd: HWND; Msg: Integer;
  wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  OldWndProc, LeftOffset, TopOffset: Integer;
  WndRect, ParentRect, ParentClientRect: TRect;
  TmpCanvas: TCanvas;
  X1, Y1, X2, Y2: Integer;
begin
  OldWndProc := GetWindowLong(hWnd, GWL_USERDATA);
  Result := CallWindowProc(Pointer(OldWndProc), hWnd, Msg, wParam, lParam);
  if Msg = WM_PAINT then
  begin
    GetWindowRect(hWnd, WndRect);
    GetWindowRect(GetParent(hWnd), ParentRect);
    GetClientRect(GetParent(hWnd), ParentClientRect);
    TopOffset := (ParentRect.Bottom - ParentRect.Top) -
      (ParentClientRect.Bottom - ParentClientRect.Top);
    LeftOffset := (ParentRect.Right - ParentRect.Left) -
      (ParentClientRect.Right - ParentClientRect.Left);
    X1 := ParentClientRect.Left + LeftOffset div 2 - (WndRect.Left - ParentRect.Left);
    Y1 := ParentClientRect.Top + TopOffset -
      (WndRect.Top - ParentRect.Top) - LeftOffset div 2;
    X2 := X1 + (ParentClientRect.Right - ParentClientRect.Left);
    Y2 := Y1 + (ParentClientRect.Bottom - ParentClientRect.Top);
    TmpCanvas := TCanvas.Create;
    try
      TmpCanvas.Handle := GetDC(hWnd);
      TmpCanvas.Pen.Color := clRed;
      TmpCanvas.Pen.Width := 4;
      TmpCanvas.MoveTo(X1, Y1);
      TmpCanvas.LineTo(X2, Y2);
    finally
      ReleaseDC(hWnd, TmpCanvas.Handle);
      TmpCanvas.Free;
    end;
  end;
end;

procedure TForm1.ButtonClick(Sender: TObject);
begin
  ReleaseButtons;
  GenerateButtons;
  Invalidate;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  DoubleBuffered := True;
  GenerateButtons;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  ReleaseButtons;
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  Canvas.Pen.Color := clRed;
  Canvas.Pen.Width := 4;
  Canvas.MoveTo(0, 0);
  Canvas.LineTo(ClientWidth, ClientHeight);  
end;

procedure TForm1.FormResize(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to 19 do
    ButtonsData[I].Invalidate;
  Invalidate;
end;

procedure TForm1.GenerateButtons;
var
  I: Integer;
begin
  Randomize;
  for I := 0 to 19 do
  begin
    ButtonsData[I] := TButton.Create(Self);
    ButtonsData[I].Parent := Self;
    ButtonsData[I].Left := Random(ClientWidth - ButtonsData[I].Width);
    ButtonsData[I].Top := Random(ClientHeight - ButtonsData[I].Height);
    ButtonsData[I].Caption := 'Button' + IntToStr(I + 1);
    ButtonsData[I].OnClick := ButtonClick;
    SetWindowLong(ButtonsData[I].Handle,
      GWL_USERDATA, GetWindowLong(ButtonsData[I].Handle, GWL_WNDPROC));
    SetWindowLong(ButtonsData[I].Handle,
      GWL_WNDPROC, Integer(@ButtonSubclassProc));
  end;
end;

procedure TForm1.ReleaseButtons;
var
  I: Integer;
begin
  for I := 0 to 19 do
    ButtonsData[I].Free;
end;

end.
